--[[--###############################
-------------REQUIRE-----------------
--]]--###############################

state = require("/include/stateswitcher")
flux = require("/include/flux")
cron = require("/include/cron")

--[[--###############################
-------------FUNCTIONS---------------
--]]--###############################

local function distanceBetween(x1, y1, x2, y2)
  return math.sqrt(math.pow(x1-x2,2)+math.pow(y1-y2, 2))
end

local function debugText(str)
  text[#text+1] = str
end

local function drawAngleRectangle( mode, x, y, w, h, a, ox, oy )
  ox = ox or 0
  oy = oy or 0
  a = a or 0
  love.graphics.push()
  love.graphics.translate( x, y )
  love.graphics.rotate( a )
  love.graphics.rectangle( mode,-ox, -oy, w, h )
  love.graphics.pop()
end

local function degreeToRadian(a)
  return math.pi * a / 180 
end

local function HSL(h, s, l, a)
  if s<=0 then return l,l,l,a end
  h, s, l = h/256*6, s/255, l/255
  local c = (1-math.abs(2*l-1))*s
  local x = (1-math.abs(h%2-1))*c
  local m,r,g,b = (l-.5*c), 0,0,0
  if h < 1     then r,g,b = c,x,0
  elseif h < 2 then r,g,b = x,c,0
  elseif h < 3 then r,g,b = 0,c,x
  elseif h < 4 then r,g,b = 0,x,c
  elseif h < 5 then r,g,b = x,0,c
  else              r,g,b = c,0,x
  end return (r+m)*255,(g+m)*255,(b+m)*255,a
end

--[[--###############################
-------------VARIABLES---------------
--]]--###############################

text = {"Console started"}

local debugMode = true
local hueMode = false

local hue = 0

local speed = 4
local gravity = 3000

--initial graphics setup
love.graphics.setBackgroundColor(127, 178, 240) --set the background color
objects = {} -- table to hold all our physical objects

objects.slide = {direction = 0, triangleSize = 50/math.tan(degreeToRadian(45))}

objects.slide.main = {
  x = WWIDTH*0.5,
  y = WHEIGHT*0.6,
  width = 600,
  height = 100,
  xVel = 0
}

objects.slide.sideLeft = {
  x = objects.slide.main.x-objects.slide.main.width*0.5,
  y = objects.slide.main.y,
  p1 = {x = objects.slide.main.x-objects.slide.main.width*0.5-objects.slide.triangleSize, y = objects.slide.main.y},
  p2 = {x = objects.slide.main.x-objects.slide.main.width*0.5, y = objects.slide.main.y-objects.slide.main.height*0.5},
  p3 = {x = objects.slide.main.x-objects.slide.main.width*0.5, y = objects.slide.main.y+objects.slide.main.height*0.5}
}

objects.slide.sideRight = {
  x = objects.slide.main.x+objects.slide.main.width*0.5,
  y = objects.slide.main.y,
  p1 = {x = objects.slide.main.x+objects.slide.main.width*0.5+objects.slide.triangleSize, y = objects.slide.main.y},
  p2 = {x = objects.slide.main.x+objects.slide.main.width*0.5, y = objects.slide.main.y-objects.slide.main.height*0.5},
  p3 = {x = objects.slide.main.x+objects.slide.main.width*0.5, y = objects.slide.main.y+objects.slide.main.height*0.5}
}

objects.slide.bar = {
  x = objects.slide.main.x,
  y = objects.slide.main.y,
  speed = 5,
  width = 50,
  height = objects.slide.main.height
}

objects.player = {
  x = WWIDTH*0.5,
  yOrigin = objects.slide.main.y-objects.slide.main.height*0.5-25,
  y = objects.slide.main.y-objects.slide.main.height*0.5-25,
  width = 50,
  jumpheight = 900,
  yVel = 0,
  direction = 0,
  rotation = 0
}

local directionTimer = cron.every(0.5, function() objects.player.direction = math.random(0, 1)*2-1 end)

function love.update(dt)
  flux.update(dt)
  
  directionTimer:update(dt)

  --here we are going to create some keyboard events
  if love.keyboard.isDown("left") then
    objects.slide.direction = -1
  elseif love.keyboard.isDown("right") then
    objects.slide.direction = 1
  else
    objects.slide.direction = 0
  end
  
  local xVel = objects.player.direction+objects.slide.direction* (objects.player.y~=objects.player.yOrigin and 0 or 1)
  
  objects.player.x = objects.player.x + xVel * speed
  objects.slide.bar.x = objects.slide.bar.x + objects.slide.direction * objects.slide.bar.speed
  if objects.slide.bar.x > objects.slide.main.x+objects.slide.main.width*0.5-objects.slide.bar.width*0.5 then
    objects.slide.bar.x = objects.slide.main.x-objects.slide.main.width*0.5+objects.slide.bar.width*0.5
  elseif objects.slide.bar.x < objects.slide.main.x-objects.slide.main.width*0.5+objects.slide.bar.width*0.5 then
    objects.slide.bar.x = objects.slide.main.x+objects.slide.main.width*0.5-objects.slide.bar.width*0.5
  end
  
  if love.keyboard.isDown(" ") then
    if objects.player.y == objects.player.yOrigin then
      local effect = "quad"
      flux.to(objects.player, 0.25, {y = objects.player.yOrigin-150, rotation = objects.player.rotation+degreeToRadian(45)*objects.slide.direction}):ease(effect.."out"):after(0.25, {y = objects.player.yOrigin, rotation = objects.player.rotation+degreeToRadian(90)*objects.slide.direction}):ease(effect.."in")
    end
    --[[
    if objects.player.yVel == 0 then -- we're probably on the ground, let's jump
      objects.player.yVel = objects.player.jumpheight
    end
    ]]--
  end
  
  --[[
  if objects.player.yVel ~= 0 then -- we're probably jumping
    objects.player.y = objects.player.y - objects.player.yVel * dt
    objects.player.yVel = objects.player.yVel - gravity * dt
    if objects.player.y > objects.player.yOrigin then -- we hit the ground again
      objects.player.yVel = 0
      objects.player.y = objects.player.yOrigin
    end
  end
  ]]--
  
  if debugMode == true then
    while #text > 40 do
      table.remove(text, 1)
    end
  end
  
  if hue > 255 then hue = 0
  elseif hue < 0 then hue = 255 end
  hue = hue +100*dt
  
end

function love.draw()
  
  love.graphics.setColor(255, 255, 255) -- player
  drawAngleRectangle("fill", objects.player.x, objects.player.y, objects.player.width, objects.player.width, objects.player.rotation, objects.player.width*0.5, objects.player.width*0.5)
  
  if hueMode then love.graphics.setColor(HSL(hue, 100, 100)) else love.graphics.setColor(78, 122, 199) end -- slide middle
  love.graphics.rectangle("fill", objects.slide.main.x-objects.slide.main.width*0.5, objects.slide.main.y-objects.slide.main.height*0.5, objects.slide.main.width, objects.slide.main.height)
  
  if objects.player.direction == -1 then
    if hueMode then love.graphics.setColor(HSL(hue, 80, 80)) else love.graphics.setColor(53, 71, 140) end
  else
    if hueMode then love.graphics.setColor(HSL(hue, 100, 100)) else love.graphics.setColor(78, 122, 199) end -- slide middle
  end
  love.graphics.polygon("fill", {objects.slide.sideLeft.p1.x, objects.slide.sideLeft.p1.y, objects.slide.sideLeft.p2.x, objects.slide.sideLeft.p2.y, objects.slide.sideLeft.p3.x, objects.slide.sideLeft.p3.y})
  
  if objects.player.direction == 1 then
    if hueMode then love.graphics.setColor(HSL(hue, 80, 80)) else love.graphics.setColor(53, 71, 140) end
  else
    if hueMode then love.graphics.setColor(HSL(hue, 100, 100)) else love.graphics.setColor(78, 122, 199) end -- slide middle
  end
  love.graphics.polygon("fill", {objects.slide.sideRight.p1.x, objects.slide.sideRight.p1.y, objects.slide.sideRight.p2.x, objects.slide.sideRight.p2.y, objects.slide.sideRight.p3.x, objects.slide.sideRight.p3.y})

  love.graphics.setColor(53, 71, 140) -- direction indicator
  love.graphics.rectangle("fill", objects.slide.bar.x-objects.slide.bar.width*0.5, objects.slide.bar.y-objects.slide.bar.height*0.5, objects.slide.bar.width, objects.slide.bar.height)

  if hueMode then love.graphics.setBackgroundColor(HSL(hue, 42, 42)) end -- background

  if debugMode == true then
  love.graphics.setColor(0, 0, 0)
  love.graphics.circle("fill", objects.player.x, objects.player.y, 2)
  love.graphics.circle("fill", objects.slide.main.x, objects.slide.main.y, 2)
  love.graphics.circle("fill", objects.slide.sideLeft.x, objects.slide.sideLeft.y, 2)
  love.graphics.circle("fill", objects.slide.sideRight.x, objects.slide.sideRight.y, 2)
  love.graphics.circle("fill", objects.slide.bar.x, objects.slide.bar.y, 2)
    for i = 1,#text do
        love.graphics.setColor(255,255,255, 255 - (i-1) * 6)
        love.graphics.print(text[#text - (i-1)], 10, i * 15)
    end
  end
end

function love.keypressed(key)
  if key == " " then
    if objects.player.yVel == 0 then -- we're probably on the ground, let's jump
      objects.player.yVel = objects.player.jumpheight
    end
  end
end

function love.keyreleased(key)

end
